
#' @title load_primary_netcdf
#' @description Load the primary netcdf file (`simu_results_0.nc`) in the given path
#' 
#' @param path The path to the directory containing camp simulations netcdf results
#' 
#' @return an object of class `ncdf4`
#' 
load_primary_netcdf <- function(path) {
  nc_file <- paste0(path, "/simu_results_0.nc")
  return(nc_file)
}

#' @title load_secondary_netcdf
#' @description Load secondary netcdf files (any `.nc` file except `simu_results_0.nc`) in the given path
#' They are picked in a random order, and the number of loaded files is limited by `max_to_take`
#' 
#' @param path The path to the directory containing camp simulations netcdf results
#' @param max_to_take (default = 112) maximum number of files to load
#' 
#' @return a list of objects of class `ncdf4`
#' 
load_secondary_netcdf <- function(path, max_to_take = 112) {
  file_list <- dir(
    path,
    pattern = glob2rx("simu_results_*.nc"),
    full.names = TRUE
  )

  # and randomly pick simulations except the first one ("simu_results_0")
  file_list <- file_list[2:(min(length(file_list) - 1, max_to_take))]
  
  ncfile_list <- file_list

  return(ncfile_list)
}


#' @title get_gas_names
#' @description read the primary netcdf output file and outputs a data.table
#' matching `gas_species_dim` with `gas_species_name`
#' 
#' @param path The path to the directory containing camp simulations netcdf results
#' 
#' @return a data.table object with two columns: `gas_species_dim` `gas_species_name`
#' 
get_gas_names <- function(path) {
  nc_file <- load_primary_netcdf(path)
  gas_names <- CustomReadNetCDF(nc_file, vars = "gas_species_name")
  return(gas_names)
}

#' @title list_gas_names
#' @description get the list of gas phase species
#' 
#' @param path  The path to the directory containing camp simulations netcdf results
#' 
#' @return a character vector of species names
#' @export
list_gas_names <- function(path) {
  gas_names <- get_gas_names(path)
  return(gas_names$gas_species_name)
}

#' @title get_var_from_primary_cell
#' 
#' @param path The path to the directory containing camp simulations netcdf results
#' @param varname The name of the variable to read
#' 
#' @return a data.table
#' @export
get_var_from_primary_cell <- function(path, varname) {
  ncfile <- load_primary_netcdf(path)
  dt <- metR::ReadNetCDF(ncfile, vars = varname, key = TRUE)
  return(dt)
}


#' @title get_var_from_all_secondary_cells
#' 
#' @param path The path to the directory containing camp simulations netcdf results
#' @param varname The name of the variable to read
#' @param max_to_take (default = 112) maximum number of files to load
#' 
#' @return a data.table
#' @export
get_var_from_all_secondary_cells <- function(path, varname, max_to_take = 112) {

  ncfile_list <- load_secondary_netcdf(path, max_to_take - 1)

  dt_list <- lapply(
      ncfile_list, function(x) {
      metR::ReadNetCDF(x, vars = varname, key = TRUE)
    }
  )
  dt <- do.call("rbind", dt_list)

  return(dt)
}

#' @title get_var_from_all_cells
#' 
#' @param path The path to the directory containing camp simulations netcdf results
#' @param varname The name of the variable to read
#' @param max_to_take (default = 112) maximum number of files to load
#' 
#' @return a data.table
#' @export
get_var_from_all_cells <- function(path, varname, max_to_take = 112) {
  primary_dt <- get_var_from_primary_cell(path, varname)
  secondary_dt <- get_var_from_all_secondary_cells(path, varname, max_to_take)
  result <- rbind(primary_dt, secondary_dt)

  return(result)
}


#' @title get_gas_species_concentrations
#' @description outputs the gas phase concentration of the selected species for all cells in given path
#' 
#' @param path The path to the directory containing camp simulations netcdf results
#' @param species_list the list of gas phase species we want to get
#' @param max_to_take (default = 112) maximum number of files to load
#' 
#' @return a data.table object
#' 
#' @export
get_gas_species_concentrations <- function(path, species_list, max_to_take = 112) {
  gas_names <- get_gas_names(path)
  dt <- get_var_from_all_cells(path, "gas_species_concentrations", max_to_take)
  dt[
    gas_names,
    on = c("gas_species_dim"),
    species := gas_species_name
  ]

  return (dt[species %in% species_list])
}